/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

This is an example Callback Object used with Search / Query, specifically with the 
DKDatastoreICM.executeWithCallback() function.  This is not a runnable Sample, but 
instead an object used directly by the SSearchICM as the callback object used.

Callback Object
    A callback object is any object that implements the dkCallback interface.  This
    is an example of one such implementation.  Please refer to the Application
    Programming Reference (javadoc) for complete information on the interface.

    Data Callback Function
        The server calls this function when a new set of results are processed.  Sets are 
        returned in a set of results specified by the threshold.  The User-Program is free
        to do as it wishes with the new set.  This sample object will simply add them to
        a results collection (as a java.util.Vector).  Other useful operations could be to
        add them to a listing in a UI of an application as they become available instead
        of waiting until all are retrieved.  

        When the server calls this function to report a new set of results, it checks the
        return variable specified to see if the user cancelled the operation.  If the
        return value is found to be 'true', the server cancels its operation.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.util.Vector;

/************************************************************************************************
 *          FILENAME: SSearchCallbackObjectICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Example Callback Object used by SSearchICM.  Not a runnable Sample.  Used
 *                    by other samples.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Example Callback Object
 *                    dkCallback interface
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: Not a runnable Sample.  Not a program.  Example object used by SSearchICM.
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: None
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SSearchCallbackObjectICM implements dkCallback{

    // Internal Variables
    private boolean userRequestsCancel;     // Set to true when the object's cancelQuery() oeration is called by the User-Program
    private boolean serverIsDone;           // Set to true when the server indicates that there are no more results to be returned.
    private int     threshold;              // Size of sets to be returned by server.  This is to be specified by the user.
    private Vector  results;                // Vector containing DKDDOs returned as results.
    
    // Error Detection Variables
    private DKException firstException;     // First failure / exception caught, null if no errors

   /**
    * Constructor for the Example Callback Object, implementing the dkCallback interface.
    * By default retrieved 5 hits at a time.
    **/
    public SSearchCallbackObjectICM(){
        userRequestsCancel = false;     
        serverIsDone       = false;
        threshold          = 5;      // Default is 5 for this example object.
        results            = new Vector();
        firstException     = null;
    }

    //==================================================================================
    // dkCallback Interface
    //==================================================================================
    // Set Threshold
    //----------------------------------------------------------------------------------
   /**
    * Sets the number of returned hits for each data callback. This method should be 
    * set by the User-Applications.
    * @param nHitsThreshold  The number of hits in the result collection for each callback.
    **/
    public void setThreshold(int nHitsThreshold){
        threshold = nHitsThreshold;
    }
    
    //----------------------------------------------------------------------------------
    // Get Threshold
    //----------------------------------------------------------------------------------
   /**
    * Gets the number of returned hits that the server returns for each callback. The 
    * datastore will call this method for each query.
    * @return  Returns the threshold value.
    **/
    public int getThreshold(){
        return(threshold);        
    }

    //----------------------------------------------------------------------------------
    // Cancel Query
    //----------------------------------------------------------------------------------
   /**
    * Indicate that the User-Program has canceled the query. It should be called by the 
    * User-Application to let the server know to stop the query.
    **/
    public void cancelQuery(){
        userRequestsCancel = true;
    }

    //----------------------------------------------------------------------------------
    // Data Callback
    //----------------------------------------------------------------------------------
   /**
    * The server calls this function when a new set of results is processed.  Sets are 
    * returned in a set of results specified by the threshold.  The User-Program is free
    * to do as it wishes with the new set.  This sample object will simply add them to
    * a results collection (as a java.util.Vector).  Other useful operations could be to
    * add them to a listing in a UI of an application as they become available instead
    * of waiting until all are retrieved.  
    *
    * When the server calls this function to report a new set of results, it checks the
    * return variable specified to see if the user cancelled the operation.  If the
    * return value is found to be 'true', the server cancels its operation.
    *
    * @param ddoList           A collection of DKDDO objects.
    * @param resultsComplete   Indicates whether the server is done with the query 
    *                          or if there are more hits.  If true, this function 
    *                          will not be called again by the server.  If false,
    *                          more results may be returned.
    * @return  Returns true if the query is done or if the user has canceled the query.  If
    *          this value is false, the server will not retrieve any further results after
    *          calling this method.
    **/
    public boolean dataCallback(dkCollection ddoList, boolean resultsComplete){

        serverIsDone = resultsComplete; // Save the indication of whether or not the server is done.

        try{
            // Process New Set
            if(ddoList!=null){ // Make sure some results exist.  The server may have discovered that no addition results exist, and is just notifying that it is complete.
                dkIterator iter = ddoList.createIterator();   // Create an iterator to go through the results.
                while(iter.more()){                           // While there are still items to iterate through, continue.
                    DKDDO ddo = (DKDDO) iter.next();          // Move pointer to next element & obtain that object.   
                    results.add(ddo);                         // Add ddo to results vector.
                    System.out.println("Callback Object:  Processed Item '"+((DKPidICM)ddo.getPidObject()).getItemId()+"'  ("+ddo.getPidObject().getObjectType()+")");
                }
            }
        } catch(DKException e) {
            SConnectDisconnectICM.printException(e);
            if(firstException==null)    // If it is the first failure, save it
                firstException = e;     // Store the first exception so that it may be detected by the User-Program
        }
        
        if(userRequestsCancel) // If the user requests to cancel the query, report to Server.
            return(true);
        else                   // Otherwise, return whether or not the server has indicated if it is done.
            return(serverIsDone);
    }

   /**
    * The server calls this function to report an exception encountered within the
    * backend datastore.  Errors encountered outside of the dataCallback() function
    * use this function as the means to notify the callback object of the event.
    * The callback object may choose how to continue.  Some examples of actions may
    * include simply doing nothing, ignoring the error, or re-issuing the query.
    *
    * The server may continue returning results after this error.  If stopping the
    * query is desired, be sure to notify the server by the return code of the 
    * dataCallback() function.  Since an error occurred, it is likely that the
    * server will have stopped further callback operation for the query and will
    * not actually call the dataCallback() function again.
    *
    * @param 	exc an exception from the backend datastore
    **/
    public void reportException(DKException exc){
        SConnectDisconnectICM.printException(exc);
        if(firstException==null)    // If it is the first failure, save it
            firstException = exc;   // Store the first exception so that it may be detected by the User-Program
    }

    //==================================================================================
    // Helper Functions Used in This Object
    //==================================================================================
    // Is Done
    //----------------------------------------------------------------------------------
   /**
    *  Determine if the Query operations have completed.
    *  @return  Returns true when done, false if not.
    **/
    public boolean isDone(){
        return(serverIsDone);
    }

    //----------------------------------------------------------------------------------
    // Get Results
    //----------------------------------------------------------------------------------
   /**
    * Obtain a reference to the current results Vector storing all DDOs retrieved so
    * far.
    * @Return  Returns a reference to the Vector storing all DDOs retrieve so far.
    **/
    public java.util.Vector getResults(){
        return(results);   
    }

    //----------------------------------------------------------------------------------
    // Error Detected
    //----------------------------------------------------------------------------------
   /**
    * Determine if an exception was caught when dataCallback() called by the server.
    * @return  Returns true if any exceptions have been caught.
    **/
    public boolean error(){
        if(firstException==null)  // If not exceptions caught yet, return false.
            return(false);
        else                     // Otherwise, at least one error has been detected.
            return(true);
    }

    //----------------------------------------------------------------------------------
    // Get First Exception
    //----------------------------------------------------------------------------------
   /**
    * Returns the first failure / exception caught by dataCallback() or null if none.
    * @Return  Returns the first exception caught by dataCallback() or null if none.
    **/
    public DKException getFirstException(){
        return(firstException);   
    }

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SSearchCallbackObjectICM
              
